
// Titles.java

/* Apisitt Rattana, Jan 14, 2001
   Computer Engineering, Prince of Songkla University
   Revised by
     Andrew Davison, dandrew@ratree.psu.ac.th, Feb 2001
*/
/*
  The Titles database is built from a text file (usualy titlesDB.txt)
  which contains titles information of the form:
     Title: The Might Banana
	 Issues 2 4 56-67 1002
	 imageURL: http://foobar.com/ban***.htm
	 $

  Issues can be single numbers or ranges.

  The image URL contains *'s which are replaced by the issue
  number before being displayed. The *'s are left-padded with 0's,
  so issue 2 becomes ban002.htm. Numbers which are bigger than the
  number of stars are inserted without padding, so issue 1002 becomes
  ban1002.htm

  Groups of titles at a single site begin with 3 fields:
    sourceID: AD
    sourceName: Andrew's Banana Collection
    sourceURL: http://foobar.com/~ad/banana/

  This information is recorded in a separate SourceMap object which
  maps source IDs to name/URL pairs. This information is used in the
  "Sources" combo box and the Sources column of the table.

  A collection of issues with the same title, but different image
  URLs can be grouped without repeating the title:
     Title: The Might Banana
	 Issues 2 4 56-67 1002
	 imageURL: http://foobar.com/ban***.htm
	 $
     Issues: 1005-1222
     imageURL: http://narfoo.com/ban***.html 
     $


  I know this sounds complicated, but the best thing is to look
  at the titlesDB.txt data file. It seems straight forward to me :)

*/

import java.io.*;
import java.util.*;


public class Titles
{
  private Vector titlesInfo;    // a vector of entries
  private SourceMap sm;         // maps source IDs to name/URL pairs

  public Titles(String dbName)
  // read in the titles database from a file (dbName)
  { 
    titlesInfo = new Vector();
	sm = new SourceMap();

	String line;
    String title, issues, imageURL;
	String sourceID, sourceName, sourceURL;
    boolean entryEnded = true;   // used to detect missing $
	int numSourceFields = 3;     // used to detect missing source field

    imageURL = null; title = null; issues = null;
	sourceID = null; sourceName = null; sourceURL = null;

    try {
      BufferedReader br = 
	        new BufferedReader( new FileReader(dbName));
      line = br.readLine();
      while (line != null) {
        if (line.startsWith("\n"))			// skip blank lines
          ;
        if (line.startsWith("//"))			// skip comment lines
          ;
        else if (line.startsWith("$")) {	// end of entry
		  Entry e = new Entry(title, issues, imageURL, sourceID);
          titlesInfo.add(e);  // add title entry
          // Reset issue and URL values before processing the next title
		  // do not reset title; it may be reused
          issues = null; imageURL = null;
		  entryEnded = true;
        }
        else if (line.startsWith("Title: "))
          title = line.substring(7).trim();
		else if (line.startsWith("Issues: ")) {
		  issues = line.substring(8).trim();
		  if (numSourceFields != 3) {
		    System.out.println("Warning: missing source field in " +
			                    dbName + " above title/issue " + 
								title + "/" + issues);
		    numSourceFields = 3;  // reset
		  }
    	  if (!entryEnded)
	        System.out.println("Warning: missing $ in " + dbName +
			         " before title/issue " + 
					 title + "/" + issues);
		  entryEnded = false;   // new entry beginning
		}
        else if (line.startsWith("imageURL: "))
          imageURL = line.substring(10).trim();

        else if (line.startsWith("sourceID: ")) {
          sourceID = line.substring(10).trim();
		  sourceName = null; sourceURL = null;  // reset source info
		  numSourceFields = 1;  // new source info starting
		}
        else if (line.startsWith("sourceName: ")) {
          sourceName = line.substring(12).trim();
		  numSourceFields++;
		}
        else if (line.startsWith("sourceURL: ")) {
          sourceURL = line.substring(11).trim();
		  sm.add(sourceID, sourceName, sourceURL);  // add source info
		  numSourceFields++;
		}

		line = br.readLine();
      } // end of while
	}
    catch ( IOException e ) {
      System.out.println("IO Exception: " + e);
    }

  } // end of Titles() constructor


  public SourceMap getSourceMap()
  {  return sm;  }


  public String toString()   // for debugging
  {  return titlesInfo.toString(); }


  public Vector findEntries(String title, int issueNo, ComicSearch cs)
  // Return a vector of entries which matches the title and issue number.
  {
    int i = 0;
    int size = titlesInfo.size();
    Vector results = new Vector();

    while(( i < size)  && !cs.stopSearchPressed() ){
      if (((Entry)titlesInfo.get(i)).matches(title, issueNo))
        results.addElement( (Entry) titlesInfo.get(i) );
      i++;
    }
    return results;
  } // end of findEntries()

} // end of Titles class
